@@ -47,6 +47,10 @@ def test_serialize():
          np.ones(shape=(5,), dtype=('f8', 32)),
          np.ones(shape=(5,), dtype=[('x', 'f8', 32)]),
          np.array([(1, 'abc')], dtype=[('x', 'i4'), ('s', object)]),
+         np.zeros(5000, dtype=[('x%d'%i,'<f8') for i in range(4)]),
+         np.zeros(5000, dtype='S32'),
+         np.zeros((1, 1000, 1000)),
+         np.arange(12)[::2],  # non-contiguous array
          np.ones(shape=(5, 6)).astype(dtype=[('total', '<f8'), ('n', '<f8')])])
 def test_dumps_serialize_numpy(x):
     header, frames = serialize(x)
@@ -136,4 +140,5 @@ def test_dont_compress_uncompressable_data():
     x = np.ones(100)
     header, [data] = serialize(x)
     assert 'compression' not in header
-    assert data.obj.ctypes.data == x.ctypes.data
+    if isinstance(data, memoryview):
+        assert data.obj.ctypes.data == x.ctypes.data
