@@ -16,6 +16,7 @@ def slow_raise_value_error(condition, duration=0.05):
         raise ValueError("condition evaluated to True")
 
 
+@pytest.mark.skip(reason="intermittent blocking failures")
 @pytest.mark.parametrize('joblib', joblibs)
 def test_simple(loop, joblib):
     if joblib is None:
@@ -23,7 +24,7 @@ def test_simple(loop, joblib):
     Parallel = joblib.Parallel
     delayed = joblib.delayed
     with cluster() as (s, [a, b]):
-        with joblib.parallel_backend('distributed', loop=loop,
+        with joblib.parallel_backend('dask.distributed', loop=loop,
                 scheduler_host=('127.0.0.1', s['port'])):
 
             seq = Parallel()(delayed(inc)(i) for i in range(10))
@@ -51,7 +52,7 @@ def test_dont_assume_function_purity(loop, joblib):
     Parallel = joblib.Parallel
     delayed = joblib.delayed
     with cluster() as (s, [a, b]):
-        with joblib.parallel_backend('distributed', loop=loop,
+        with joblib.parallel_backend('dask.distributed', loop=loop,
                 scheduler_host=('127.0.0.1', s['port'])):
 
             x, y = Parallel()(delayed(random2)() for i in range(2))
