@@ -3,7 +3,6 @@ from __future__ import print_function, division, absolute_import
 from datetime import datetime
 import os
 import sys
-from time import time
 
 import pytest
 from toolz import valmap
@@ -12,7 +11,8 @@ from tornado.iostream import StreamClosedError
 from tornado import gen
 
 from distributed import Nanny, rpc, Scheduler
-from distributed.core import connect, read, write
+from distributed.core import connect, read, write, close
+from distributed.metrics import time
 from distributed.protocol.pickle import dumps, loads
 from distributed.utils import ignoring
 from distributed.utils_test import gen_cluster
@@ -59,8 +59,8 @@ def test_str(s, a, b):
     assert str(a.ncores) in repr(a)
 
 
-@gen_cluster(ncores=[], timeout=20)
-def test_nanny_process_failure(s):
+@gen_cluster(ncores=[], timeout=20, client=True)
+def test_nanny_process_failure(c, s):
     n = Nanny(s.ip, s.port, ncores=2, ip='127.0.0.1', loop=s.loop)
     yield n._start()
     first_dir = n.worker_dir
@@ -71,9 +71,7 @@ def test_nanny_process_failure(s):
     ww = rpc(ip=n.ip, port=n.worker_port)
     yield ww.update_data(data=valmap(dumps, {'x': 1, 'y': 2}))
     with ignoring(StreamClosedError):
-        yield ww.compute(function=dumps(sys.exit),
-                         args=dumps((0,)),
-                         key='z')
+        yield c._run(sys.exit, 0, workers=[n.worker_address])
 
     start = time()
     while n.process is original_process:  # wait while process dies
@@ -100,6 +98,10 @@ def test_nanny_process_failure(s):
     s.stop()
 
 
+def test_nanny_no_port():
+    _ = str(Nanny('127.0.0.1', 8786))
+
+
 @gen_cluster(ncores=[])
 def test_monitor_resources(s):
     pytest.importorskip('psutil')
@@ -120,7 +122,7 @@ def test_monitor_resources(s):
         assert isinstance(msg, dict)
         assert {'cpu_percent', 'memory_percent'}.issubset(msg)
 
-    stream.close()
+    close(stream)
     yield n._close()
     s.stop()
 
@@ -137,3 +139,16 @@ def test_run(s):
         assert response['result'] == 1
 
     yield n._close()
+
+
+
+@gen_cluster(Worker=Nanny,
+             ncores=[('127.0.0.1', 1)],
+             worker_kwargs={'reconnect': False})
+def test_close_on_disconnect(s, w):
+    yield s.close()
+
+    start = time()
+    while w.status != 'closed':
+        yield gen.sleep(0.01)
+        assert time() < start + 9
