@@ -19,7 +19,7 @@ from tornado import gen
 
 import pytest
 
-from distributed import Nanny, Worker
+from distributed import Nanny, Worker, Client
 from distributed.core import connect, read, write, close, rpc
 from distributed.scheduler import validate_state, Scheduler, BANDWIDTH
 from distributed.client import _wait, _first_completed
@@ -27,7 +27,7 @@ from distributed.metrics import time
 from distributed.protocol.pickle import dumps
 from distributed.worker import dumps_function, dumps_task
 from distributed.utils_test import (inc, ignoring, dec, gen_cluster, gen_test,
-        loop, readone, slowinc, slowadd)
+        loop, readone, slowinc, slowadd, cluster, div)
 from distributed.utils import All
 from distributed.utils_test import slow
 from dask.compatibility import apply
@@ -972,3 +972,86 @@ def test_no_worker_to_memory_restrictions(c, s, a, b):
     assert s.task_state[z.key] == 'processing'
 
     yield w._close()
+
+
+def test_run_on_scheduler_sync(loop):
+    def f(dask_scheduler=None):
+        return dask_scheduler.address
+
+    with cluster() as (s, [a, b]):
+        with Client(('127.0.0.1', s['port']), loop=loop) as c:
+            address = c.run_on_scheduler(f)
+            ip, port = address.split(':')
+            assert int(port) == int(s['port'])
+
+            with pytest.raises(ZeroDivisionError):
+                c.run_on_scheduler(div, 1, 0)
+
+
+@gen_cluster(client=True)
+def test_run_on_scheduler(c, s, a, b):
+    def f(dask_scheduler=None):
+        return dask_scheduler.address
+
+    response = yield c._run_on_scheduler(f)
+    assert response == s.address
+
+
+@gen_cluster(client=True)
+def test_close_worker(c, s, a, b):
+    assert len(s.workers) == 2
+
+    yield s.close_worker(worker=a.address)
+
+    assert len(s.workers) == 1
+    assert a.address not in s.workers
+
+    yield gen.sleep(0.5)
+
+    assert len(s.workers) == 1
+
+
+@slow
+@gen_cluster(client=True, Worker=Nanny, timeout=20)
+def test_close_nanny(c, s, a, b):
+    assert len(s.workers) == 2
+    old_process = a.process
+
+    assert old_process.is_alive()
+    yield s.close_worker(worker=a.worker_address)
+
+    assert len(s.workers) == 1
+    assert a.worker_address not in s.workers
+    assert not old_process.is_alive()
+
+    for i in range(10):
+        yield gen.sleep(0.1)
+        assert len(s.workers) == 1
+
+        assert not old_process.is_alive()
+        assert not a.process
+
+
+@gen_cluster(client=True, timeout=20)
+def test_retire_workers_close(c, s, a, b):
+    yield s.retire_workers(close=True)
+    assert not s.workers
+
+
+@gen_cluster(client=True, timeout=20, Worker=Nanny)
+def test_retire_nannies_close(c, s, a, b):
+    processes = [a.process, b.process]
+    yield s.retire_workers(close=True, remove=True)
+    assert not s.workers
+
+    start = time()
+    while True:
+        if not any(proc.is_alive() for proc in processes):
+            break
+        else:
+            yield gen.sleep(0.1)
+        assert time() < start + 5
+
+    yield gen.sleep(1)
+    assert not any(proc.is_alive() for proc in processes)
+    assert not a.process and not b.process
