@@ -16,6 +16,8 @@ def test_pack_data():
 
 @gen_cluster()
 def test_gather_from_workers_permissive(s, a, b):
+    while not a.batched_stream:
+        yield gen.sleep(0.01)
     a.update_data(data={'x': 1})
     with pytest.raises(KeyError):
         yield gather_from_workers({'x': [a.address], 'y': [b.address]})
