@@ -25,8 +25,8 @@ from distributed.protocol import to_serialize
 from distributed.protocol.pickle import dumps, loads
 from distributed.sizeof import sizeof
 from distributed.worker import Worker, error_message, logger, TOTAL_MEMORY
-from distributed.utils import ignoring
-from distributed.utils_test import (loop, inc, mul, gen_cluster,
+from distributed.utils import ignoring, tmpfile
+from distributed.utils_test import (loop, inc, mul, gen_cluster, div,
         slow, slowinc, throws, current_loop, gen_test, readone)
 
 
@@ -446,8 +446,8 @@ def test_clean(c, s, a, b):
 
     yield y._result()
 
-    collections = [a.tasks, a.task_state, a.response, a.data, a.nbytes,
-                   a.durations, a.priorities]
+    collections = [a.tasks, a.task_state, a.startstops, a.data, a.nbytes,
+                   a.durations, a.priorities, a.types, a.threads]
     for c in collections:
         assert c
 
@@ -507,43 +507,12 @@ def test_system_monitor(s, a, b):
 
 
 
-@pytest.mark.skipif(not sys.platform.startswith('linux'),
-                    reason="Need 127.0.0.2 to mean localhost")
 @gen_cluster(client=True, ncores=[('127.0.0.1', 2, {'resources': {'A': 1}}),
-                                  ('127.0.0.2', 1)])
+                                  ('127.0.0.1', 1)])
 def test_restrictions(c, s, a, b):
-    # Worker restrictions
-    x = c.submit(inc, 1, workers=a.address)
-    yield x._result()
-    assert a.host_restrictions == {}
-    assert a.worker_restrictions == {x.key: {a.address}}
-    assert a.resource_restrictions == {}
-
-    yield c._cancel(x)
-
-    while x.key in a.task_state:
-        yield gen.sleep(0.01)
-
-    assert a.worker_restrictions == {}
-
-    # Host restrictions
-    x = c.submit(inc, 1, workers=['127.0.0.1'])
-    yield x._result()
-    assert a.host_restrictions == {x.key: {'127.0.0.1'}}
-    assert a.worker_restrictions == {}
-    assert a.resource_restrictions == {}
-    yield c._cancel(x)
-
-    while x.key in a.task_state:
-        yield gen.sleep(0.01)
-
-    assert a.host_restrictions == {}
-
     # Resource restrictions
     x = c.submit(inc, 1, resources={'A': 1})
     yield x._result()
-    assert a.host_restrictions == {}
-    assert a.worker_restrictions == {}
     assert a.resource_restrictions == {x.key: {'A': 1}}
     yield c._cancel(x)
 
@@ -566,3 +535,119 @@ def test_clean_nbytes(c, s, a, b):
 
     yield gen.sleep(1)
     assert len(a.nbytes) + len(b.nbytes) == 1
+
+
+@gen_cluster(client=True, ncores=[('127.0.0.1', 1)] * 20)
+def test_gather_many_small(c, s, a, *workers):
+    a.total_connections = 2
+    futures = yield c._scatter(list(range(100)))
+
+    assert all(w.data for w in workers)
+
+    def f(*args):
+        return 10
+
+    future = c.submit(f, *futures, workers=a.address)
+    yield _wait(future)
+
+    types = list(pluck(0, a.log))
+    req = [i for i, t in enumerate(types) if t == 'request-dep']
+    recv = [i for i, t in enumerate(types) if t == 'receive-dep']
+    assert min(recv) > max(req)
+
+    assert a.comm_nbytes == 0
+
+
+@gen_cluster(client=True, ncores=[('127.0.0.1', 1)] * 3)
+def test_multiple_transfers(c, s, w1, w2, w3):
+    x = c.submit(inc, 1, workers=w1.address)
+    y = c.submit(inc, 2, workers=w2.address)
+    z = c.submit(add, x, y, workers=w3.address)
+
+    yield _wait(z)
+
+    r = w3.startstops[z.key]
+    transfers = [t for t in r if t[0] == 'transfer']
+    assert len(transfers) == 2
+
+
+@gen_cluster(client=True, ncores=[('127.0.0.1', 1)] * 3)
+def test_share_communication(c, s, w1, w2, w3):
+    x = c.submit(mul, b'1', int(w3.target_message_size + 1), workers=w1.address)
+    y = c.submit(mul, b'2', int(w3.target_message_size + 1), workers=w2.address)
+    yield _wait([x, y])
+    yield c._replicate([x, y], workers=[w1.address, w2.address])
+    z = c.submit(add, x, y, workers=w3.address)
+    yield _wait(z)
+    assert len(w3.incoming_transfer_log) == 2
+    assert w1.outgoing_transfer_log
+    assert w2.outgoing_transfer_log
+
+
+@gen_cluster(client=True)
+def test_dont_overlap_communications_to_same_worker(c, s, a, b):
+    x = c.submit(mul, b'1', int(b.target_message_size + 1), workers=a.address)
+    y = c.submit(mul, b'2', int(b.target_message_size + 1), workers=a.address)
+    yield _wait([x, y])
+    z = c.submit(add, x, y, workers=b.address)
+    yield _wait(z)
+    assert len(b.incoming_transfer_log) == 2
+    l1, l2 = b.incoming_transfer_log
+
+    assert l1['stop'] < l2['start']
+
+
+@pytest.mark.avoid_travis
+@gen_cluster(client=True)
+def test_log_exception_on_failed_task(c, s, a, b):
+    with tmpfile() as fn:
+        fh = logging.FileHandler(fn)
+        try:
+            from distributed.worker import logger
+            logger.addHandler(fh)
+
+            future = c.submit(div, 1, 0)
+            yield _wait(future)
+
+            yield gen.sleep(0.1)
+            fh.flush()
+            with open(fn) as f:
+                text = f.read()
+
+            assert "ZeroDivisionError" in text
+            assert "Exception" in text
+        finally:
+            logger.removeHandler(fh)
+
+
+@gen_cluster(client=True)
+def test_clean_up_dependencies(c, s, a, b):
+    x = delayed(inc)(1)
+    y = delayed(inc)(2)
+    xx = delayed(inc)(x)
+    yy = delayed(inc)(y)
+    z = delayed(add)(xx, yy)
+
+    zz = c.persist(z)
+    yield _wait(zz)
+
+    start = time()
+    while len(a.data) + len(b.data) > 1:
+        yield gen.sleep(0.01)
+        assert time() < start + 2
+
+    assert set(a.data) | set(b.data) == {zz.key}
+
+
+@gen_cluster(client=True)
+def test_hold_onto_dependents(c, s, a, b):
+    x = c.submit(inc, 1, workers=a.address)
+    y = c.submit(inc, x, workers=b.address)
+    yield _wait(y)
+
+    assert x.key in b.data
+
+    yield c._cancel(y)
+    yield gen.sleep(0.1)
+
+    assert x.key in b.data
