@@ -308,7 +308,6 @@ class WorkerBase(Server):
         # logger.info("Finish job %d, %s", i, key)
         raise gen.Return(result)
 
-
     def run(self, stream, function, args=(), kwargs={}):
         return run(self, stream, function=function, args=args, kwargs=kwargs)
 
@@ -667,7 +666,7 @@ def weight(k, v):
 
 
 @gen.coroutine
-def run(worker, stream, function, args=(), kwargs={}, is_coro=False, wait=True):
+def run(server, stream, function, args=(), kwargs={}, is_coro=False, wait=True):
     assert wait or is_coro, "Combination not supported"
     function = loads(function)
     if args:
@@ -675,7 +674,9 @@ def run(worker, stream, function, args=(), kwargs={}, is_coro=False, wait=True):
     if kwargs:
         kwargs = loads(kwargs)
     if has_arg(function, 'dask_worker'):
-        kwargs['dask_worker'] = worker
+        kwargs['dask_worker'] = server
+    if has_arg(function, 'dask_scheduler'):
+        kwargs['dask_scheduler'] = server
     logger.info("Run out-of-band function %r", funcname(function))
     try:
         result = function(*args, **kwargs)
@@ -740,7 +741,7 @@ class Worker(WorkerBase):
     * **services:** ``{str: Server}``:
         Auxiliary web servers running on this worker
     * **service_ports:** ``{str: port}``:
-    * **total_connections**": ``int``
+    * **total_connections**: ``int``
         The maximum number of concurrent connections we want to see
     * **total_comm_nbytes**: ``int``
     * **batched_stream**: ``BatchedSend(IOstream)``
@@ -1013,7 +1014,7 @@ class Worker(WorkerBase):
                 if state in ('memory', 'error'):
                     if state == 'memory':
                         assert key in self.data
-                    logger.debug("Asked to compute prexisting result: %s: %s" ,
+                    logger.debug("Asked to compute pre-existing result: %s: %s" ,
                                  key, state)
                     self.send_task_state_to_scheduler(key)
                     return
